﻿using System;
using System.Data.Entity;
using System.Data.Spatial;
using System.Globalization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;

namespace GeoLocation
{
    [TestClass]
    public class GeoLocationTests
    {


        //private TestContext testContextInstance;

        ///// <summary>
        /////Gets or sets the test context which provides
        /////information about and functionality for the current test run.
        /////</summary>
        //public TestContext TestContext
        //{
        //    get
        //    {
        //        return testContextInstance;
        //    }
        //    set
        //    {
        //        testContextInstance = value;
        //    }
        //}

        //#region Additional test attributes
        ////
        //// You can use the following additional attributes as you write your tests:
        ////
        //// Use ClassInitialize to run code before running the first test in the class
        [ClassInitialize()]
        public static void MyClassInitialize(TestContext testContext)
        {
            Database.SetInitializer<GeoLocationContext>(new DropCreateDatabaseIfModelChanges<GeoLocationContext>());
        }

        [TestMethod]
        public void AddLocationsToDataBase()
        {
            var context = new GeoLocationContext();

            // remove all
            context.Locations.ToList().ForEach(loc => context.Locations.Remove(loc));
            context.SaveChanges();

            var location = new GeoLocation()
            {
                // Create a point using native DbGeography Factory method
                Location = DbGeography.PointFromText(
                            string.Format("POINT({0} {1})", -121.527200, 45.712113)
                            , 4326),
                Address = "301 15th Street, Hood River"
            };
            context.Locations.Add(location);

            location = new GeoLocation()
            {
                Location = CreatePoint(45.714240, -121.517265),
                Address = "The Hatchery, Bingen"
            };
            context.Locations.Add(location);

            location = new GeoLocation()
            {
                // Create a point using a helper function (lat/long)
                Location = CreatePoint(45.708457, -121.514432),
                Address = "Kaze Sushi, Hood River"
            };
            context.Locations.Add(location);

            location = new GeoLocation()
            {
                Location = CreatePoint(45.722780, -120.209227),
                Address = "Arlington, OR"
            };
            context.Locations.Add(location);

            context.SaveChanges();
        }

        [TestMethod]
        public void QueryLocationsTest()
        {
            var sourcePoint = CreatePoint(45.712113, -121.527200);

            var context = new GeoLocationContext();

            // find any locations within 5 kilometers ordered by distance
            var matches =
                context.Locations
                        .Where(loc => loc.Location.Distance(sourcePoint) < 5000)
                        .OrderBy(loc => loc.Location.Distance(sourcePoint))
                        .Select(loc => new { Address = loc.Address, Distance = loc.Location.Distance(sourcePoint) });

            Assert.IsTrue(matches.Count() > 0);

            foreach (var location in matches)
            {
                Console.WriteLine("{0} ({1:n0} meters)", location.Address, location.Distance);
            }
        }

        [TestMethod]
        public void QueryLocationsMilesTest()
        {
            var sourcePoint = CreatePoint(45.712113, -121.527200);

            var context = new GeoLocationContext();

            // find any locations within 5 miles ordered by distance
            var fiveMiles = GeoUtils.MilesToMeters(5);

            var matches =
                context.Locations
                        .Where(loc => loc.Location.Distance(sourcePoint) <= fiveMiles)
                        .OrderBy(loc => loc.Location.Distance(sourcePoint))
                        .Select(loc => new { Address = loc.Address, Distance = loc.Location.Distance(sourcePoint) });

            Assert.IsTrue(matches.Count() > 0);

            foreach (var location in matches)
            {
                Console.WriteLine("{0} ({1:n1} miles)", location.Address, GeoUtils.MetersToMiles(location.Distance));
            }
        }

        [TestMethod]
        public void RawSqlEfAddTest()
        {
            string sqlFormat =
            @"insert into GeoLocations( Location, Address) values
            ( geography::STGeomFromText('POINT({0} {1})', 4326),@p0 )";

            var sql = string.Format(sqlFormat, -121.527200, 45.712113);

            Console.WriteLine(sql);

            var context = new GeoLocationContext();

            Assert.IsTrue(context.Database.ExecuteSqlCommand(sql, "301 N. 15th Street") > 0);
        }

        [TestMethod]
        public void RawSqlEfQueryTest()
        {
            var sqlFormat =
            @"
    DECLARE @s geography
    SET @s = geography:: STGeomFromText('POINT({0} {1})' , 4326);

    SELECT     
        Address,        
        Location.ToString() as GeoString,        
        @s.STDistance( Location) as Distance        
    FROM GeoLocations
    ORDER BY Distance";

            var sql = string.Format(sqlFormat, -121.527200, 45.712113);

            var context = new GeoLocationContext();
            var locations = context.Database.SqlQuery<ResultData>(sql);

            Assert.IsTrue(locations.Count() > 0);

            foreach (var location in locations)
            {
                Console.WriteLine(location.Address + " " + location.GeoString + " " + location.Distance);
            }
        }


        public static DbGeography CreatePoint(double latitude, double longitude)
        {
            var text = string.Format(CultureInfo.InvariantCulture.NumberFormat,
                                     "POINT({0} {1})", longitude, latitude);
            // 4326 is most common coordinate system used by GPS/Maps
            return DbGeography.PointFromText(text, 4326);
        }

        /// <summary>
        /// Convert meters to miles
        /// </summary>
        /// <param name="meters"></param>
        /// <returns></returns>
        public static double MetersToMiles(double? meters)
        {
            if (meters == null)
                return 0F;

            return meters.Value * 0.000621371192;
        }



    }

    public class GeoLocationContext : DbContext
    {
        public DbSet<GeoLocation> Locations { get; set; }
    }

    public class GeoLocation
    {
        public int Id { get; set; }
        public string Name { get; set; }
        public DbGeography Location { get; set; }
        public string Address { get; set; }
    }

    public class ResultData
    {
        public string GeoString { get; set; }
        public double Distance { get; set; }
        public string Address { get; set; }
    }
}
